@@ -1,23 +1,18 @@
-"""
-Base class for directed graphs.
-
-"""
-__author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
-                            'Pieter Swart (swart@lanl.gov)',
-                            'Dan Schult(dschult@colgate.edu)'])
-#    Copyright (C) 2004-2009 by
+"""Base class for directed graphs."""
+#    Copyright (C) 2004-2011 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-#
-
+from copy import deepcopy
 import networkx as nx
 from networkx.classes.graph import Graph
-from networkx.exception import NetworkXException, NetworkXError
+from networkx.exception import NetworkXError
 import networkx.convert as convert
-from copy import deepcopy
+__author__ = """\n""".join(['Aric Hagberg (hagberg@lanl.gov)',
+                            'Pieter Swart (swart@lanl.gov)',
+                            'Dan Schult(dschult@colgate.edu)'])
 
 class DiGraph(Graph):
     """
@@ -42,8 +37,6 @@ class DiGraph(Graph):
         NetworkX graph object.  If the corresponding optional Python
         packages are installed the data can also be a NumPy matrix
         or 2d ndarray, a SciPy sparse matrix, or a PyGraphviz graph.
-    name : string, optional (default='')
-        An optional name for the graph.
     attr : keyword arguments, optional (default= no attributes)
         Attributes to add to graph as key=value pairs.
 
@@ -122,8 +115,9 @@ class DiGraph(Graph):
     >>> G.node[1]
     {'time': '5pm'}
     >>> G.node[1]['room'] = 714
+    >>> del G.node[1]['room'] # remove attribute
     >>> G.nodes(data=True)
-    [(1, {'room': 714, 'time': '5pm'}), (3, {'time': '2pm'})]
+    [(1, {'time': '5pm'}), (3, {'time': '2pm'})]
 
     Warning: adding a node to G.node does not add it to the graph.
 
@@ -171,7 +165,7 @@ class DiGraph(Graph):
 
     For details on these and other miscellaneous methods, see below.
     """
-    def __init__(self, data=None, name='', **attr):
+    def __init__(self, data=None, **attr):
         """Initialize a graph with edges, name, graph attributes.
 
         Parameters
@@ -216,11 +210,9 @@ class DiGraph(Graph):
 
         # attempt to load graph with data
         if data is not None:
-            convert.from_whatever(data,create_using=self)
+            convert.to_networkx_graph(data,create_using=self)
         # load graph attributes (must be after convert)
         self.graph.update(attr)
-
-        self.name=name
         self.edge=self.adj
 
 
@@ -282,6 +274,7 @@ class DiGraph(Graph):
         else: # update attr even if node already exists
             self.node[n].update(attr_dict)
 
+
     def add_nodes_from(self, nodes, **attr):
         """Add multiple nodes.
 
@@ -463,9 +456,9 @@ class DiGraph(Graph):
         -----
         Adding an edge that already exists updates the edge data.
 
-        NetworkX algorithms designed for weighted graphs use as
-        the edge weight a numerical value assigned to the keyword
-        'weight'.
+        Many NetworkX algorithms designed for weighted graphs use as
+        the edge weight a numerical value assigned to a keyword
+        which by default is 'weight'.
 
         Examples
         --------
@@ -670,14 +663,14 @@ class DiGraph(Graph):
         neighbors_iter() and successors_iter() are the same.
         """
         try:
-            return iter(self.succ[n].keys())
+            return iter(self.succ[n])
         except KeyError:
             raise NetworkXError("The node %s is not in the digraph."%(n,))
 
     def predecessors_iter(self,n):
         """Return an iterator over predecessor nodes of n."""
         try:
-            return iter(self.pred[n].keys())
+            return iter(self.pred[n])
         except KeyError:
             raise NetworkXError("The node %s is not in the digraph."%(n,))
 
@@ -723,6 +716,7 @@ class DiGraph(Graph):
         Notes
         -----
         Nodes in nbunch that are not in the graph will be (quietly) ignored.
+        For directed graphs this returns the out-edges.
 
         Examples
         --------
@@ -739,7 +733,7 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.adj.items())
+            nodes_nbrs=self.adj.items()
         else:
             nodes_nbrs=((n,self.adj[n]) for n in self.nbunch_iter(nbunch))
         if data:
@@ -776,7 +770,7 @@ class DiGraph(Graph):
         edges_iter : return an iterator of edges
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.pred.items())
+            nodes_nbrs=self.pred.items()
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
         if data:
@@ -797,7 +791,7 @@ class DiGraph(Graph):
         """
         return list(self.in_edges_iter(nbunch, data))
 
-    def degree_iter(self, nbunch=None, weighted=False):
+    def degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, degree).
 
         The node degree is the number of edges adjacent to the node.
@@ -807,8 +801,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -836,18 +833,18 @@ class DiGraph(Graph):
                 ((n,self.succ[n]) for n in self.nbunch_iter(nbunch)),
                 ((n,self.pred[n]) for n in self.nbunch_iter(nbunch)))
 
-        if weighted:                        
+        if weight is None:
+            for (n,succ),(n2,pred) in nodes_nbrs:
+                yield (n,len(succ)+len(pred))
+        else:
         # edge weighted graph - degree is sum of edge weights
             for (n,succ),(n2,pred) in nodes_nbrs:
-               yield (n, 
-                      sum((succ[nbr].get('weight',1) for nbr in succ))+
-                      sum((pred[nbr].get('weight',1) for nbr in pred)))
-        else:
-            for (n,succ),(n2,pred) in nodes_nbrs:
-                yield (n,len(succ)+len(pred)) 
+               yield (n,
+                      sum((succ[nbr].get(weight,1) for nbr in succ))+
+                      sum((pred[nbr].get(weight,1) for nbr in pred)))
 
 
-    def in_degree_iter(self, nbunch=None, weighted=False):
+    def in_degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, in-degree).
 
         The node in-degree is the number of edges pointing in to the node.
@@ -857,8 +854,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -880,20 +880,20 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.pred.items())
+            nodes_nbrs=self.pred.items()
         else:
             nodes_nbrs=((n,self.pred[n]) for n in self.nbunch_iter(nbunch))
 
-        if weighted:
-        # edge weighted graph - degree is sum of edge weights
-            for n,nbrs in nodes_nbrs:
-                yield (n, sum(data.get('weight',1) for data in nbrs.values()))
-        else:
+        if weight is None:
             for n,nbrs in nodes_nbrs:
                 yield (n,len(nbrs))
+        else:
+        # edge weighted graph - degree is sum of edge weights
+            for n,nbrs in nodes_nbrs:
+                yield (n, sum(data.get(weight,1) for data in nbrs.values()))
 
 
-    def out_degree_iter(self, nbunch=None, weighted=False):
+    def out_degree_iter(self, nbunch=None, weight=None):
         """Return an iterator for (node, out-degree).
 
         The node out-degree is the number of edges pointing out of the node.
@@ -903,8 +903,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -926,20 +929,20 @@ class DiGraph(Graph):
 
         """
         if nbunch is None:
-            nodes_nbrs=iter(self.succ.items())
+            nodes_nbrs=self.succ.items()
         else:
             nodes_nbrs=((n,self.succ[n]) for n in self.nbunch_iter(nbunch))
 
-        if weighted:
-        # edge weighted graph - degree is sum of edge weights
-            for n,nbrs in nodes_nbrs:
-                yield (n, sum(data.get('weight',1) for data in nbrs.values()))
-        else:
+        if weight is None:
             for n,nbrs in nodes_nbrs:
                 yield (n,len(nbrs))
+        else:
+        # edge weighted graph - degree is sum of edge weights
+            for n,nbrs in nodes_nbrs:
+                yield (n, sum(data.get(weight,1) for data in nbrs.values()))
 
 
-    def in_degree(self, nbunch=None, weighted=False):
+    def in_degree(self, nbunch=None, weight=None):
         """Return the in-degree of a node or nodes.
 
         The node in-degree is the number of edges pointing in to the node.
@@ -949,8 +952,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -974,11 +980,11 @@ class DiGraph(Graph):
         [0, 1]
         """
         if nbunch in self:      # return a single node
-            return next(self.in_degree_iter(nbunch,weighted=weighted))[1]
+            return next(self.in_degree_iter(nbunch,weight))[1]
         else:           # return a dict
-            return dict(self.in_degree_iter(nbunch,weighted=weighted))
+            return dict(self.in_degree_iter(nbunch,weight))
 
-    def out_degree(self, nbunch=None, weighted=False):
+    def out_degree(self, nbunch=None, weight=None):
         """Return the out-degree of a node or nodes.
 
         The node out-degree is the number of edges pointing out of the node.
@@ -988,8 +994,11 @@ class DiGraph(Graph):
         nbunch : iterable container, optional (default=all nodes)
             A container of nodes.  The container will be iterated
             through once.
-        weighted : bool, optional (default=False)
-           If True return the sum of edge weights adjacent to the node.
+
+        weight : string or None, optional (default=None)
+           The edge attribute that holds the numerical value used 
+           as a weight.  If None, then each edge has weight 1.
+           The degree is the sum of the edge weights adjacent to the node.
 
         Returns
         -------
@@ -1011,9 +1020,9 @@ class DiGraph(Graph):
 
         """
         if nbunch in self:      # return a single node
-            return next(self.out_degree_iter(nbunch,weighted=weighted))[1]
+            return next(self.out_degree_iter(nbunch,weight))[1]
         else:           # return a dict
-            return dict(self.out_degree_iter(nbunch,weighted=weighted))
+            return dict(self.out_degree_iter(nbunch,weight))
 
     def clear(self):
         """Remove all nodes and edges from the graph.
@@ -1031,7 +1040,6 @@ class DiGraph(Graph):
         []
 
         """
-        self.name=''
         self.succ.clear()
         self.pred.clear()
         self.node.clear()
@@ -1085,9 +1093,15 @@ class DiGraph(Graph):
         """
         return deepcopy(self)
 
-    def to_undirected(self):
+    def to_undirected(self, reciprocal=False):
         """Return an undirected representation of the digraph.
 
+        Parameters
+        ----------
+        reciprocal : bool (optional)
+          If True only keep edges that appear in both directions 
+          in the original digraph. 
+
         Returns
         -------
         G : Graph
@@ -1115,14 +1129,19 @@ class DiGraph(Graph):
 
         See the Python copy module for more information on shallow
         and deep copies, http://docs.python.org/library/copy.html.
-
         """
         H=Graph()
         H.name=self.name
         H.add_nodes_from(self)
-        H.add_edges_from( (u,v,deepcopy(d))
-                          for u,nbrs in self.adjacency_iter()
-                          for v,d in nbrs.items() )
+        if reciprocal is True:
+            H.add_edges_from( (u,v,deepcopy(d))
+                              for u,nbrs in self.adjacency_iter()
+                              for v,d in nbrs.items() 
+                              if v in self.pred[u])
+        else:
+            H.add_edges_from( (u,v,deepcopy(d))
+                              for u,nbrs in self.adjacency_iter()
+                              for v,d in nbrs.items() )
         H.graph=deepcopy(self.graph)
         H.node=deepcopy(self.node)
         return H
@@ -1143,11 +1162,11 @@ class DiGraph(Graph):
         """
         if copy:
             H = self.__class__(name="Reverse of (%s)"%self.name)
-            H.pred=self.succ.copy()
-            H.adj=self.pred.copy()
-            H.succ=H.adj
-            H.graph=self.graph.copy()
-            H.node=self.node.copy()
+            H.add_nodes_from(self)
+            H.add_edges_from( (v,u,deepcopy(d)) for u,v,d 
+                              in self.edges(data=True) )
+            H.graph=deepcopy(self.graph)
+            H.node=deepcopy(self.node)
         else:
             self.pred,self.succ=self.succ,self.pred
             self.adj=self.succ
@@ -1197,14 +1216,15 @@ class DiGraph(Graph):
         bunch = self.nbunch_iter(nbunch)
         # create new graph and copy subgraph into it
         H = self.__class__()
-        H.name = "Subgraph of (%s)"%(self.name)
+        # copy node and attribute dictionaries
+        for n in bunch:
+            H.node[n]=self.node[n]
         # namespace shortcuts for speed
         H_succ=H.succ
         H_pred=H.pred
         self_succ=self.succ
-        self_pred=self.pred
         # add nodes
-        for n in bunch:
+        for n in H:
             H_succ[n]={}
             H_pred[n]={}
         # add edges
@@ -1215,8 +1235,5 @@ class DiGraph(Graph):
                     # add both representations of edge: u-v and v-u
                     Hnbrs[v]=datadict
                     H_pred[v][u]=datadict
-        # copy node and attribute dictionaries
-        for n in H:
-            H.node[n]=self.node[n]
         H.graph=self.graph
         return H
