@@ -3,7 +3,7 @@
 Generators for random graphs.
 
 """
-#    Copyright (C) 2004-2010 by 
+#    Copyright (C) 2004-2011 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
@@ -22,7 +22,6 @@ from collections import defaultdict
 
 __all__ = ['fast_gnp_random_graph',
            'gnp_random_graph',
-           'directed_gnp_random_graph',
            'dense_gnm_random_graph',
            'gnm_random_graph',
            'erdos_renyi_graph',
@@ -44,13 +43,8 @@ __all__ = ['fast_gnp_random_graph',
 #-------------------------------------------------------------------------
 
 
-def fast_gnp_random_graph(n, p, create_using=None, seed=None):
-    """Return a random graph G_{n,p}.
-
-    The G_{n,p} graph choses each of the possible [n(n-1)]/2 edges
-    with probability p.
-
-    Sometimes called Erdős-Rényi graph, or binomial graph.
+def fast_gnp_random_graph(n, p, seed=None, directed=False):
+    """Return a random graph G_{n,p} (Erdős-Rényi graph, binomial graph).
 
     Parameters
     ----------
@@ -58,55 +52,77 @@ def fast_gnp_random_graph(n, p, create_using=None, seed=None):
         The number of nodes.
     p : float
         Probability for edge creation.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
+    directed : bool, optional (default=False)
+        If True return a directed graph 
       
     Notes
     -----
+    The G_{n,p} graph algorithm chooses each of the [n(n-1)]/2
+    (undirected) or n(n-1) (directed) possible edges with probability p.
+
     This algorithm is O(n+m) where m is the expected number of
     edges m=p*n*(n-1)/2.
     
-    It should be faster than gnp_random_graph when p is small, and
-    the expected number of edges is small, (sparse graph).
+    It should be faster than gnp_random_graph when p is small and
+    the expected number of edges is small (sparse graph).
+
+    See Also
+    --------
+    gnp_random_graph
 
     References
     ----------
-    .. [1] Batagelj and Brandes,
+    .. [1] Vladimir Batagelj and Ulrik Brandes, 
        "Efficient generation of large random networks",
        Phys. Rev. E, 71, 036113, 2005.
     """
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(n,create_using)
+    G = empty_graph(n)
     G.name="fast_gnp_random_graph(%s,%s)"%(n,p)
 
     if not seed is None:
         random.seed(seed)
 
-    v=1  # Nodes in graph are from 0,n-1 (this is the second node index).
-    w=-1
-    lp=math.log(1.0-p)  
+    if p <= 0 or p >= 1:
+        return nx.gnp_random_graph(n,p,directed=directed)
 
-    while v<n:
-        lr=math.log(1.0-random.random())
-        w=w+1+int(lr/lp)
-        while w>=v and v<n:
-            w=w-v
-            v=v+1
-        if v<n:
-            G.add_edge(v,w)
+    v = 1  # Nodes in graph are from 0,n-1 (this is the second node index).
+    w = -1
+    lp = math.log(1.0 - p)  
+
+    if directed:
+        G=nx.DiGraph(G)
+        while v < n:
+            lr = math.log(1.0 - random.random())
+            w = w + 1 + int(lr/lp)
+            if v == w: # avoid self loops
+                w = w + 1
+            while  w >= n and v < n:
+                w = w - n
+                v = v + 1
+                if v == w: # avoid self loops
+                    w = w + 1
+            if v < n:
+                G.add_edge(v, w)
+    else:
+        while v < n:
+            lr = math.log(1.0 - random.random())
+            w = w + 1 + int(lr/lp)
+            while w >= v and v < n:
+                w = w - v
+                v = v + 1
+            if v < n:
+                G.add_edge(v, w)
     return G
 
 
-def gnp_random_graph(n, p, create_using=None, seed=None):
-    """Return a random graph G_{n,p}.
+def gnp_random_graph(n, p, seed=None, directed=False):
+    """Return a random graph G_{n,p} (Erdős-Rényi graph, binomial graph).
 
-    Choses each of the possible [n(n-1)]/2 edges with probability p.
-    This is the same as binomial_graph and erdos_renyi_graph. 
+    Chooses each of the possible edges with probability p.
 
-    Sometimes called Erdős-Rényi graph, or binomial graph.
+    This is also called binomial_graph and erdos_renyi_graph. 
 
     Parameters
     ----------
@@ -114,38 +130,47 @@ def gnp_random_graph(n, p, create_using=None, seed=None):
         The number of nodes.
     p : float
         Probability for edge creation.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
-      
-    See Also
+    directed : bool, optional (default=False)
+        If True return a directed graph 
+    
+    See Also  
     --------
-    fast_gnp_random_graph()
+    fast_gnp_random_graph
 
     Notes
     -----
     This is an O(n^2) algorithm.  For sparse graphs (small p) see
-    fast_gnp_random_graph. 
+    fast_gnp_random_graph for a faster algorithm.
 
     References
     ----------
     .. [1] P. Erdős and A. Rényi, On Random Graphs, Publ. Math. 6, 290 (1959).
     .. [2] E. N. Gilbert, Random Graphs, Ann. Math. Stat., 30, 1141 (1959).
-
     """
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(n,create_using)
+    if directed:
+        G=nx.DiGraph()
+    else:
+        G=nx.Graph()
+    G.add_nodes_from(range(n))
     G.name="gnp_random_graph(%s,%s)"%(n,p)
+    if p<=0:
+        return G
+    if p>=1:
+        return complete_graph(n,create_using=G)
 
     if not seed is None:
         random.seed(seed)
 
-    for u in range(n):
-        for v in range(u+1,n):
-            if random.random() < p:
-                G.add_edge(u,v)
+    if G.is_directed():
+        edges=itertools.permutations(range(n),2)
+    else:
+        edges=itertools.combinations(range(n),2)
+
+    for e in edges:
+        if random.random() < p:        
+            G.add_edge(*e)
     return G
 
 
@@ -153,55 +178,7 @@ def gnp_random_graph(n, p, create_using=None, seed=None):
 binomial_graph=gnp_random_graph
 erdos_renyi_graph=gnp_random_graph
 
-def directed_gnp_random_graph(n, p, create_using=None, seed=None):
-    """Return a directed random graph.
-
-    Chooses each of the possible n(n-1) edges with probability p.
-
-    This is a directed version of G_np.
-
-    Parameters
-    ----------
-    n : int
-        The number of nodes.
-    p : float
-        Probability for edge creation.
-    create_using :  graph, optional (default DiGraph)
-        Use specified graph as a container.
-    seed : int, optional
-        Seed for random number generator (default=None). 
-      
-    See Also
-    --------
-    gnp_random_graph()
-    fast_gnp_random_graph()
-
-    Notes
-    -----
-    This is an O(n^2) algorithm.  
-
-    References
-    ----------
-    .. [1] P. Erdős and A. Rényi, On Random Graphs, Publ. Math. 6, 290 (1959).
-    .. [2] E. N. Gilbert, Random Graphs, Ann. Math. Stat., 30, 1141 (1959).
-
-    """
-    if create_using is None:
-        create_using=nx.DiGraph()
-    G=empty_graph(n,create_using)
-    G.name="directed gnp_random_graph(%s,%s)"%(n,p)
-
-    if not seed is None:
-        random.seed(seed)
-
-    for u in range(n):
-        for v in range(n):
-            if u==v: continue
-            if random.random() < p:
-                G.add_edge(u,v)
-    return G
-
-def dense_gnm_random_graph(n, m, create_using=None, seed=None):
+def dense_gnm_random_graph(n, m, seed=None):
     """Return the random graph G_{n,m}.
 
     Gives a graph picked randomly out of the set of all graphs
@@ -214,8 +191,6 @@ def dense_gnm_random_graph(n, m, create_using=None, seed=None):
         The number of nodes.
     m : int
         The number of edges.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
       
@@ -227,25 +202,19 @@ def dense_gnm_random_graph(n, m, create_using=None, seed=None):
     -----
     Algorithm by Keith M. Briggs Mar 31, 2006.
     Inspired by Knuth's Algorithm S (Selection sampling technique),
-    in section 3.4.2 of
+    in section 3.4.2 of [1]_.
 
     References
     ----------
-    .. [1] Donald E. Knuth,
-        The Art of Computer Programming,
-        Volume 2 / Seminumerical algorithms
-        Third Edition, Addison-Wesley, 1997.
- 
+    .. [1] Donald E. Knuth, The Art of Computer Programming,
+        Volume 2/Seminumerical algorithms, Third Edition, Addison-Wesley, 1997.
     """
     mmax=n*(n-1)/2
     if m>=mmax:
-        G=complete_graph(n,create_using)
+        G=complete_graph(n)
     else:
-        if create_using is not None and create_using.is_directed():
-            raise nx.NetworkXError("Directed Graph not supported")
-        G=empty_graph(n,create_using)
-
-        G.name="dense_gnm_random_graph(%s,%s)"%(n,m)
+        G=empty_graph(n)
+    G.name="dense_gnm_random_graph(%s,%s)"%(n,m)
   
     if n==1 or m>=mmax:
         return G
@@ -268,10 +237,10 @@ def dense_gnm_random_graph(n, m, create_using=None, seed=None):
             u+=1
             v=u+1
 
-def gnm_random_graph(n, m, create_using=None, seed=None):
+def gnm_random_graph(n, m, seed=None, directed=False):
     """Return the random graph G_{n,m}.
 
-    Gives a graph picked randomly out of the set of all graphs
+    Produces a graph picked randomly out of the set of all graphs
     with n nodes and m edges.
 
     Parameters
@@ -280,15 +249,16 @@ def gnm_random_graph(n, m, create_using=None, seed=None):
         The number of nodes.
     m : int
         The number of edges.
-    create_using :  graph, optional (default Graph)
-        Use specified graph as a container.
     seed : int, optional
         Seed for random number generator (default=None). 
-      
+    directed : bool, optional (default=False)
+        If True return a directed graph 
     """
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(n,create_using)
+    if directed:
+        G=nx.DiGraph()
+    else:
+        G=nx.Graph()
+    G.add_nodes_from(range(n))
     G.name="gnm_random_graph(%s,%s)"%(n,m)
 
     if seed is not None:
@@ -296,9 +266,11 @@ def gnm_random_graph(n, m, create_using=None, seed=None):
 
     if n==1:
         return G
-
-    if m>=n*(n-1)/2:
-        return complete_graph(n,create_using)
+    max_edges=n*(n-1)
+    if not directed:
+        max_edges/=2.0
+    if m>=max_edges:
+        return complete_graph(n,create_using=G)
 
     nlist=G.nodes()
     edge_count=0
@@ -308,16 +280,13 @@ def gnm_random_graph(n, m, create_using=None, seed=None):
         v = random.choice(nlist)
         if u==v or G.has_edge(u,v):
             continue
-        # is this faster?
-        # (u,v)=random.sample(nlist,2)
-        #  if G.has_edge(u,v):
-        #      continue
         else:
             G.add_edge(u,v)
             edge_count=edge_count+1
     return G
 
-def newman_watts_strogatz_graph(n, k, p, create_using=None, seed=None):
+
+def newman_watts_strogatz_graph(n, k, p, seed=None):
     """Return a Newman-Watts-Strogatz small world graph.
 
     Parameters
@@ -328,8 +297,6 @@ def newman_watts_strogatz_graph(n, k, p, create_using=None, seed=None):
         Each node is connected to k nearest neighbors in ring topology
     p : float 
         The probability of adding a new edge for each edge
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional        
        seed for random number generator (default=None)
 
@@ -357,9 +324,7 @@ def newman_watts_strogatz_graph(n, k, p, create_using=None, seed=None):
         random.seed(seed)
     if k>=n // 2: 
         raise nx.NetworkXError("k>=n/2, choose smaller k or larger n")
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(n,create_using)
+    G=empty_graph(n)
     G.name="newman_watts_strogatz_graph(%s,%s,%s)"%(n,k,p)
     nlist = G.nodes()
     fromv = nlist
@@ -382,7 +347,7 @@ def newman_watts_strogatz_graph(n, k, p, create_using=None, seed=None):
     return G            
 
 
-def watts_strogatz_graph(n, k, p, create_using=None, seed=None):
+def watts_strogatz_graph(n, k, p, seed=None):
     """Return a Watts-Strogatz small-world graph.
 
 
@@ -394,8 +359,6 @@ def watts_strogatz_graph(n, k, p, create_using=None, seed=None):
         Each node is connected to k nearest neighbors in ring topology
     p : float 
         The probability of rewiring each edge 
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional        
         Seed for random number generator (default=None)
 
@@ -425,17 +388,10 @@ def watts_strogatz_graph(n, k, p, create_using=None, seed=None):
     """
     if k>=n/2: 
         raise nx.NetworkXError("k>=n/2, choose smaller k or larger n")
-    if create_using is None:
-        G = nx.Graph()
-    elif create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    else:
-        G = create_using
-        G.clear()
-
     if seed is not None:
         random.seed(seed)
 
+    G = nx.Graph()
     G.name="watts_strogatz_graph(%s,%s,%s)"%(n,k,p)
     nodes = list(range(n)) # nodes are labeled 0 to n-1
     # connect each node to k/2 neighbors
@@ -458,7 +414,7 @@ def watts_strogatz_graph(n, k, p, create_using=None, seed=None):
                 G.add_edge(u,w)
     return G            
 
-def connected_watts_strogatz_graph(n, k, p, tries=100, create_using=None, seed=None):
+def connected_watts_strogatz_graph(n, k, p, tries=100, seed=None):
     """Return a connected Watts-Strogatz small-world graph.
 
     Attempt to generate a connected realization by repeated 
@@ -475,8 +431,6 @@ def connected_watts_strogatz_graph(n, k, p, tries=100, create_using=None, seed=N
         The probability of rewiring each edge 
     tries : int
         Number of attempts to generate a connected graph.  
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional
          The seed for random number generator.
 
@@ -486,17 +440,17 @@ def connected_watts_strogatz_graph(n, k, p, tries=100, create_using=None, seed=N
     watts_strogatz_graph()
 
     """
-    G = watts_strogatz_graph(n, k, p, create_using, seed)
+    G = watts_strogatz_graph(n, k, p, seed)
     t=1
     while not nx.is_connected(G):
-        G = watts_strogatz_graph(n, k, p, create_using, seed)
+        G = watts_strogatz_graph(n, k, p, seed)
         t=t+1
         if t>tries:
             raise nx.NetworkXError("Maximum number of tries exceeded")
     return G
 
 
-def random_regular_graph(d, n, create_using=None, seed=None):
+def random_regular_graph(d, n, seed=None):
     """Return a random regular graph of n nodes each with degree d.
     
     The resulting graph G has no self-loops or parallel edges.
@@ -507,8 +461,6 @@ def random_regular_graph(d, n, create_using=None, seed=None):
       Degree
     n : integer
       Number of nodes. The value of n*d must be even.
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : hashable object
         The seed for random number generator.
 
@@ -516,7 +468,6 @@ def random_regular_graph(d, n, create_using=None, seed=None):
     -----
     The nodes are numbered form 0 to n-1.
 
-
     Kim and Vu's paper [2]_ shows that this algorithm samples in an
     asymptotically uniform way from the space of random graphs when
     d = O(n**(1/3-epsilon)).
@@ -540,14 +491,6 @@ def random_regular_graph(d, n, create_using=None, seed=None):
     if not 0 <= d < n:
         raise nx.NetworkXError("the 0 <= d < n inequality must be satisfied")
 
-    if create_using is None:
-        G = nx.Graph()
-    elif create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    else:
-        G = create_using
-        G.clear()
-
     if seed is not None:
         random.seed(seed)
 
@@ -603,13 +546,25 @@ def random_regular_graph(d, n, create_using=None, seed=None):
     while edges is None:
         edges = _try_creation()
     
+    G = nx.Graph()
     G.name = "random_regular_graph(%s, %s)" % (d, n)
     G.add_edges_from(edges)
 
     return G
 
+def _random_subset(seq,m):
+    """ Return m unique elements from seq.
 
-def barabasi_albert_graph(n, m, create_using=None, seed=None):
+    This differs from random.sample which can return repeated
+    elements if seq holds repeated elements.
+    """
+    targets=set()
+    while len(targets)<m:
+        x=random.choice(seq)
+        targets.add(x)
+    return targets
+    
+def barabasi_albert_graph(n, m, seed=None):
     """Return random graph using Barabási-Albert preferential attachment model.
         
     A graph of n nodes is grown by attaching new nodes each with m
@@ -622,8 +577,6 @@ def barabasi_albert_graph(n, m, create_using=None, seed=None):
         Number of nodes
     m : int
         Number of edges to attach from a new node to existing nodes
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional
         Seed for random number generator (default=None).   
 
@@ -644,15 +597,11 @@ def barabasi_albert_graph(n, m, create_using=None, seed=None):
     if m < 1 or  m >=n:
         raise nx.NetworkXError(\
               "Barabási-Albert network must have m>=1 and m<n, m=%d,n=%d"%(m,n))
-
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-
     if seed is not None:
         random.seed(seed)    
 
     # Add m initial nodes (m0 in barabasi-speak) 
-    G=empty_graph(m,create_using)  
+    G=empty_graph(m)
     G.name="barabasi_albert_graph(%s,%s)"%(n,m)
     # Target nodes for new edges
     targets=list(range(m))
@@ -669,14 +618,11 @@ def barabasi_albert_graph(n, m, create_using=None, seed=None):
         repeated_nodes.extend([source]*m) 
         # Now choose m unique nodes from the existing nodes 
         # Pick uniformly from repeated_nodes (preferential attachement) 
-        targets=set()
-        while len(targets)<m:
-            x=random.choice(repeated_nodes)
-            targets.add(x)
+        targets = _random_subset(repeated_nodes,m)
         source += 1
     return G
 
-def powerlaw_cluster_graph(n, m, p, create_using=None, seed=None):
+def powerlaw_cluster_graph(n, m, p, seed=None):
     """Holme and Kim algorithm for growing graphs with powerlaw
     degree distribution and approximate average clustering. 
 
@@ -688,8 +634,6 @@ def powerlaw_cluster_graph(n, m, p, create_using=None, seed=None):
         the number of random edges to add for each new node
     p : float,
         Probability of adding a triangle after adding a random edge
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional
         Seed for random number generator (default=None).   
       
@@ -725,20 +669,16 @@ def powerlaw_cluster_graph(n, m, p, create_using=None, seed=None):
     if p > 1 or p < 0:
         raise nx.NetworkXError(\
               "NetworkXError p must be in [0,1], p=%f"%(p))
-
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-
     if seed is not None:
         random.seed(seed)    
 
-    G=empty_graph(m,create_using) # add m initial nodes (m0 in barabasi-speak)
+    G=empty_graph(m) # add m initial nodes (m0 in barabasi-speak)
     G.name="Powerlaw-Cluster Graph"
     repeated_nodes=G.nodes()  # list of existing nodes to sample from
                            # with nodes repeated once for each adjacent edge 
     source=m               # next node is m
     while source<n:        # Now add the other n-1 nodes
-        possible_targets=random.sample(repeated_nodes,m)
+        possible_targets = _random_subset(repeated_nodes,m)
         # do one preferential attachment for new node
         target=possible_targets.pop()
         G.add_edge(source,target)  
@@ -765,7 +705,7 @@ def powerlaw_cluster_graph(n, m, p, create_using=None, seed=None):
         source += 1
     return G
 
-def random_lobster(n, p1, p2, create_using=None, seed=None):
+def random_lobster(n, p1, p2, seed=None):
     """Return a random lobster.
 
      A lobster is a tree that reduces to a caterpillar when pruning all
@@ -782,8 +722,6 @@ def random_lobster(n, p1, p2, create_using=None, seed=None):
         Probability of adding an edge to the backbone
     p2 : float
         Probability of adding an edge one level beyond backbone
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional
         Seed for random number generator (default=None).   
     """
@@ -791,9 +729,7 @@ def random_lobster(n, p1, p2, create_using=None, seed=None):
     if seed is not None:
         random.seed(seed)
     llen=int(2*random.random()*n + 0.5)
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    L=path_graph(llen,create_using)
+    L=path_graph(llen)
     L.name="random_lobster(%d,%s,%s)"%(n,p1,p2)
     # build caterpillar: add edges to path graph with probability p1
     current_node=llen-1
@@ -806,7 +742,7 @@ def random_lobster(n, p1, p2, create_using=None, seed=None):
                 L.add_edge(current_node-1,current_node)
     return L # voila, un lobster!
 
-def random_shell_graph(constructor, create_using=None, seed=None):
+def random_shell_graph(constructor, seed=None):
     """Return a random shell graph for the constructor given.
 
     Parameters
@@ -820,8 +756,6 @@ def random_shell_graph(constructor, create_using=None, seed=None):
     d : float
         The ratio of inter-shell (next) edges to intra-shell edges.
         d=0 means no intra shell edges, d=1 for the last shell
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional
         Seed for random number generator (default=None).   
       
@@ -831,9 +765,7 @@ def random_shell_graph(constructor, create_using=None, seed=None):
     >>> G=nx.random_shell_graph(constructor)        
 
     """
-    if create_using is not None and create_using.is_directed():
-        raise nx.NetworkXError("Directed Graph not supported")
-    G=empty_graph(0,create_using)
+    G=empty_graph(0)
     G.name="random_shell_graph(constructor)"
 
     if seed is not None:
@@ -870,7 +802,7 @@ def random_shell_graph(constructor, create_using=None, seed=None):
     return G
 
 
-def random_powerlaw_tree(n, gamma=3, create_using=None, seed=None, tries=100):
+def random_powerlaw_tree(n, gamma=3, seed=None, tries=100):
     """Return a tree with a powerlaw degree distribution.
 
     Parameters
@@ -879,8 +811,6 @@ def random_powerlaw_tree(n, gamma=3, create_using=None, seed=None, tries=100):
         The number of nodes
     gamma : float
         Exponent of the power-law
-    create_using : graph, optional (default Graph)
-        The graph instance used to build the graph.
     seed : int, optional
         Seed for random number generator (default=None).   
     tries : int
@@ -902,7 +832,7 @@ def random_powerlaw_tree(n, gamma=3, create_using=None, seed=None, tries=100):
     except:
         raise nx.NetworkXError(\
               "Exceeded max (%d) attempts for a valid tree sequence."%tries)
-    G=degree_sequence_tree(s,create_using)
+    G=degree_sequence_tree(s)
     G.name="random_powerlaw_tree(%s,%s)"%(n,gamma)
     return G
 
